@@ -101,7 +101,7 @@ def _setup_root_parser():
 
     # General bugzilla connection options
     p.add_argument('--bugzilla', default=default_url,
-            help="bugzilla XMLRPC URI. default: %s" % default_url)
+            help="bugzilla URI. default: %s" % default_url)
     p.add_argument("--nosslverify", dest="sslverify",
                  action="store_false", default=True,
                  help="Don't error on invalid bugzilla SSL certificate")
@@ -163,6 +163,21 @@ def _parser_add_output_options(p):
             help="one line summary of the bug (useful for scripts)")
     outg.add_argument('--json', action='store_const', dest='output',
             const='json', help="output contents in json format")
+    outg.add_argument("--includefield", action="append",
+            help="Pass the field name to bugzilla include_fields list. "
+                 "Only the fields passed to include_fields are returned "
+                 "by the bugzilla server. "
+                 "This can be specified multiple times.")
+    outg.add_argument("--extrafield", action="append",
+            help="Pass the field name to bugzilla extra_fields list. "
+                 "When used with --json this can be used to request "
+                 "bugzilla to return values for non-default fields. "
+                 "This can be specified multiple times.")
+    outg.add_argument("--excludefield", action="append",
+            help="Pass the field name to bugzilla exclude_fields list. "
+                 "When used with --json this can be used to request "
+                 "bugzilla to not return values for a field. "
+                 "This can be specified multiple times.")
     outg.add_argument('--raw', action='store_const', dest='output',
             const='raw', help="raw output of the bugzilla contents. This "
             "format is unstable and difficult to parse. Use --json instead.")
@@ -243,7 +258,7 @@ def _parser_add_bz_fields(rootp, command):
     # Put this at the end, so it sticks out more
     p.add_argument('--field',
         metavar="FIELD=VALUE", action="append", dest="fields",
-        help="Manually specify a bugzilla XMLRPC field. FIELD is "
+        help="Manually specify a bugzilla API field. FIELD is "
         "the raw name used by the bugzilla instance. For example, if your "
         "bugzilla instance has a custom field cf_my_field, do:\n"
         "  --field cf_my_field=VALUE")
@@ -750,7 +765,21 @@ def _bug_field_repl_cb(bz, b, matchobj):
 
 def _format_output(bz, opt, buglist):
     if opt.output in ['raw', 'json']:
-        buglist = bz.getbugs([b.bug_id for b in buglist])
+        include_fields = None
+        exclude_fields = None
+        extra_fields = None
+
+        if opt.includefield:
+            include_fields = opt.includefield
+        if opt.excludefield:
+            exclude_fields = opt.excludefield
+        if opt.extrafield:
+            extra_fields = opt.extrafield
+
+        buglist = bz.getbugs([b.bug_id for b in buglist],
+                include_fields=include_fields,
+                exclude_fields=exclude_fields,
+                extra_fields=extra_fields)
         if opt.output == 'json':
             _format_output_json(buglist)
         if opt.output == 'raw':
@@ -940,7 +969,7 @@ def _do_modify(bz, parser, opt):
     # Now for the things we can't blindly batch.
     # Being able to prepend/append to whiteboards, which are just
     # plain string values, is an old rhbz semantic that we try to maintain
-    # here. This is a bit weird for traditional bugzilla XMLRPC
+    # here. This is a bit weird for traditional bugzilla API
     log.debug("Adjusting whiteboard fields one by one")
     for bug in bz.getbugs(bugid_list):
         update_kwargs = {}
