@@ -3,68 +3,71 @@
 # Copyright (C) 2007, 2008, 2009, 2010 Red Hat Inc.
 # Author: Will Woods <wwoods@redhat.com>
 #
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
-# the full text of the license.
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
-import locale
+import copy
 from logging import getLogger
-import sys
+
+from ._util import to_encoding
+
 
 log = getLogger(__name__)
 
 
 class Bug(object):
-    '''A container object for a bug report. Requires a Bugzilla instance -
+    """
+    A container object for a bug report. Requires a Bugzilla instance -
     every Bug is on a Bugzilla, obviously.
     Optional keyword args:
         dict=DICT   - populate attributes with the result of a getBug() call
         bug_id=ID   - if dict does not contain bug_id, this is required before
                       you can read any attributes or make modifications to this
                       bug.
-    '''
+    """
     def __init__(self, bugzilla, bug_id=None, dict=None, autorefresh=False):
         # pylint: disable=redefined-builtin
         # API had pre-existing issue that we can't change ('dict' usage)
 
         self.bugzilla = bugzilla
-        self._bug_fields = []
+        self._rawdata = {}
         self.autorefresh = autorefresh
 
+        # pylint: disable=protected-access
+        self._aliases = self.bugzilla._get_bug_aliases()
+        # pylint: enable=protected-access
+
         if not dict:
             dict = {}
         if bug_id:
             dict["id"] = bug_id
 
-        log.debug("Bug(%s)", sorted(dict.keys()))
         self._update_dict(dict)
-
         self.weburl = bugzilla.url.replace('xmlrpc.cgi',
                                            'show_bug.cgi?id=%i' % self.bug_id)
 
     def __str__(self):
-        '''Return a simple string representation of this bug
+        """
+        Return a simple string representation of this bug
 
         This is available only for compatibility. Using 'str(bug)' and
         'print(bug)' is not recommended because of potential encoding issues.
         Please use unicode(bug) where possible.
-        '''
-        if hasattr(sys.version_info, "major") and sys.version_info.major >= 3:
-            return self.__unicode__()
-        else:
-            return self.__unicode__().encode(
-                locale.getpreferredencoding(), 'replace')
+        """
+        return to_encoding(self.__unicode__())
 
     def __unicode__(self):
-        '''Return a simple unicode string representation of this bug'''
-        return u"#%-6s %-10s - %s - %s" % (self.bug_id, self.bug_status,
+        """
+        Return a simple unicode string representation of this bug
+    """
+        return "#%-6s %-10s - %s - %s" % (self.bug_id, self.bug_status,
                                           self.assigned_to, self.summary)
 
     def __repr__(self):
-        return '<Bug #%i on %s at %#x>' % (self.bug_id, self.bugzilla.url,
-                                           id(self))
+        url = ""
+        if self.bugzilla:
+            url = self.bugzilla.url
+        return '<Bug #%i on %s at %#x>' % (self.bug_id, url, id(self))
 
     def __getattr__(self, name):
         refreshed = False
@@ -74,11 +77,7 @@ class Bug(object):
                 # have never been called.
                 return self.__dict__[name]
 
-            # pylint: disable=protected-access
-            aliases = self.bugzilla._get_bug_aliases()
-            # pylint: enable=protected-access
-
-            for newname, oldname in aliases:
+            for newname, oldname in self._aliases:
                 if name == oldname and newname in self.__dict__:
                     return self.__dict__[newname]
 
@@ -109,47 +108,52 @@ class Bug(object):
                     "to adjust your include_fields for getbug/query." % name)
         raise AttributeError(msg)
 
+    def get_raw_data(self):
+        """
+        Return the raw API dictionary data that has been used to
+        populate this bug
+        """
+        return copy.deepcopy(self._rawdata)
+
     def refresh(self, include_fields=None, exclude_fields=None,
         extra_fields=None):
-        '''
+        """
         Refresh the bug with the latest data from bugzilla
-        '''
+        """
         # pylint: disable=protected-access
+        extra_fields = list(self._rawdata.keys()) + (extra_fields or [])
         r = self.bugzilla._getbug(self.bug_id,
             include_fields=include_fields, exclude_fields=exclude_fields,
-            extra_fields=self._bug_fields + (extra_fields or []))
+            extra_fields=extra_fields)
         # pylint: enable=protected-access
         self._update_dict(r)
     reload = refresh
 
-    def _update_dict(self, newdict):
-        '''
-        Update internal dictionary, in a way that ensures no duplicate
-        entries are stored WRT field aliases
-        '''
+    def _translate_dict(self, newdict):
         if self.bugzilla:
             self.bugzilla.post_translation({}, newdict)
 
-            # pylint: disable=protected-access
-            aliases = self.bugzilla._get_bug_aliases()
-            # pylint: enable=protected-access
+        for newname, oldname in self._aliases:
+            if oldname not in newdict:
+                continue
 
-            for newname, oldname in aliases:
-                if oldname not in newdict:
-                    continue
+            if newname not in newdict:
+                newdict[newname] = newdict[oldname]
+            elif newdict[newname] != newdict[oldname]:
+                log.debug("Update dict contained differing alias values "
+                          "d[%s]=%s and d[%s]=%s , dropping the value "
+                          "d[%s]", newname, newdict[newname], oldname,
+                        newdict[oldname], oldname)
+            del(newdict[oldname])
 
-                if newname not in newdict:
-                    newdict[newname] = newdict[oldname]
-                elif newdict[newname] != newdict[oldname]:
-                    log.debug("Update dict contained differing alias values "
-                              "d[%s]=%s and d[%s]=%s , dropping the value "
-                              "d[%s]", newname, newdict[newname], oldname,
-                            newdict[oldname], oldname)
-                del(newdict[oldname])
 
-        for key in newdict.keys():
-            if key not in self._bug_fields:
-                self._bug_fields.append(key)
+    def _update_dict(self, newdict):
+        """
+        Update internal dictionary, in a way that ensures no duplicate
+        entries are stored WRT field aliases
+        """
+        self._translate_dict(newdict)
+        self._rawdata.update(newdict)
         self.__dict__.update(newdict)
 
         if 'id' not in self.__dict__ and 'bug_id' not in self.__dict__:
@@ -161,14 +165,15 @@ class Bug(object):
     ##################
 
     def __getstate__(self):
-        ret = {}
-        for key in self._bug_fields:
-            ret[key] = self.__dict__[key]
+        ret = self._rawdata.copy()
+        ret["_aliases"] = self._aliases
         return ret
 
     def __setstate__(self, vals):
-        self._bug_fields = []
+        self._rawdata = {}
         self.bugzilla = None
+        self._aliases = vals.get("_aliases", [])
+        self.autorefresh = False
         self._update_dict(vals)
 
 
@@ -177,12 +182,12 @@ class Bug(object):
     #####################
 
     def setstatus(self, status, comment=None, private=False):
-        '''
+        """
         Update the status for this bug report.
         Commonly-used values are ASSIGNED, MODIFIED, and NEEDINFO.
 
         To change bugs to CLOSED, use .close() instead.
-        '''
+        """
         # Note: fedora bodhi uses this function
         vals = self.bugzilla.build_update(status=status,
                                           comment=comment,
@@ -193,7 +198,8 @@ class Bug(object):
 
     def close(self, resolution, dupeid=None, fixedin=None,
               comment=None, isprivate=False):
-        '''Close this bug.
+        """
+        Close this bug.
         Valid values for resolution are in bz.querydefaults['resolution_list']
         For bugzilla.redhat.com that's:
         ['NOTABUG', 'WONTFIX', 'DEFERRED', 'WORKSFORME', 'CURRENTRELEASE',
@@ -205,14 +211,14 @@ class Bug(object):
           version that fixes the bug.
         You can optionally add a comment while closing the bug. Set 'isprivate'
           to True if you want that comment to be private.
-        '''
+        """
         # Note: fedora bodhi uses this function
         vals = self.bugzilla.build_update(comment=comment,
                                           comment_private=isprivate,
                                           resolution=resolution,
                                           dupe_of=dupeid,
                                           fixed_in=fixedin,
-                                          status="CLOSED")
+                                          status=str("CLOSED"))
         log.debug("close: update=%s", vals)
 
         return self.bugzilla.update_bugs(self.bug_id, vals)
@@ -224,7 +230,7 @@ class Bug(object):
 
     def setassignee(self, assigned_to=None,
                     qa_contact=None, comment=None):
-        '''
+        """
         Set any of the assigned_to or qa_contact fields to a new
         bugzilla account, with an optional comment, e.g.
         setassignee(assigned_to='wwoods@redhat.com')
@@ -234,7 +240,7 @@ class Bug(object):
         will throw a ValueError.
 
         Returns [bug_id, mailresults].
-        '''
+        """
         if not (assigned_to or qa_contact):
             raise ValueError("You must set one of assigned_to "
                              " or qa_contact")
@@ -247,11 +253,11 @@ class Bug(object):
         return self.bugzilla.update_bugs(self.bug_id, vals)
 
     def addcc(self, cclist, comment=None):
-        '''
+        """
         Adds the given email addresses to the CC list for this bug.
         cclist: list of email addresses (strings)
         comment: optional comment to add to the bug
-        '''
+        """
         vals = self.bugzilla.build_update(comment=comment,
                                           cc_add=cclist)
         log.debug("addcc: update=%s", vals)
@@ -259,9 +265,9 @@ class Bug(object):
         return self.bugzilla.update_bugs(self.bug_id, vals)
 
     def deletecc(self, cclist, comment=None):
-        '''
+        """
         Removes the given email addresses from the CC list for this bug.
-        '''
+        """
         vals = self.bugzilla.build_update(comment=comment,
                                           cc_remove=cclist)
         log.debug("deletecc: update=%s", vals)
@@ -274,10 +280,10 @@ class Bug(object):
     ####################
 
     def addcomment(self, comment, private=False):
-        '''
+        """
         Add the given comment to this bug. Set private to True to mark this
         comment as private.
-        '''
+        """
         # Note: fedora bodhi uses this function
         vals = self.bugzilla.build_update(comment=comment,
                                           comment_private=private)
@@ -286,9 +292,9 @@ class Bug(object):
         return self.bugzilla.update_bugs(self.bug_id, vals)
 
     def getcomments(self):
-        '''
+        """
         Returns an array of comment dictionaries for this bug
-        '''
+        """
         comment_list = self.bugzilla.get_comments([self.bug_id])
         return comment_list['bugs'][str(self.bug_id)]['comments']
 
@@ -357,33 +363,38 @@ class Bug(object):
     # Experimental methods #
     ########################
 
-    def get_attachment_ids(self):
-        # pylint: disable=protected-access
-        proxy = self.bugzilla._proxy
-        # pylint: enable=protected-access
-
+    def get_attachments(self, include_fields=None, exclude_fields=None):
+        """
+        Helper call to Bugzilla.get_attachments. If you want to fetch
+        specific attachment IDs, use that function instead
+        """
         if "attachments" in self.__dict__:
-            attachments = self.attachments
-        else:
-            rawret = proxy.Bug.attachments(
-                {"ids": [self.bug_id], "exclude_fields": ["data"]})
-            attachments = rawret["bugs"][str(self.bug_id)]
+            return self.attachments
 
-        return [a["id"] for a in attachments]
+        data = self.bugzilla.get_attachments([self.bug_id], None,
+                include_fields, exclude_fields)
+        return data["bugs"][str(self.bug_id)]
+
+    def get_attachment_ids(self):
+        """
+        Helper function to return only the attachment IDs for this bug
+        """
+        return [a["id"] for a in self.get_attachments(exclude_fields=["data"])]
 
     def get_history_raw(self):
-        '''
+        """
         Experimental. Get the history of changes for this bug.
-        '''
+        """
         return self.bugzilla.bugs_history_raw([self.bug_id])
 
 
 class User(object):
-    '''Container object for a bugzilla User.
+    """
+    Container object for a bugzilla User.
 
     :arg bugzilla: Bugzilla instance that this User belongs to.
     Rest of the params come straight from User.get()
-    '''
+    """
     def __init__(self, bugzilla, **kwargs):
         self.bugzilla = bugzilla
         self.__userid = kwargs.get('id')
@@ -435,11 +446,74 @@ class User(object):
         self.__dict__.update(newuser.__dict__)
 
     def updateperms(self, action, groups):
-        '''
+        """
         A method to update the permissions (group membership) of a bugzilla
         user.
 
         :arg action: add, remove, or set
         :arg groups: list of groups to be added to (i.e. ['fedora_contrib'])
-        '''
+        """
         self.bugzilla.updateperms(self.name, action, groups)
+
+
+class Group(object):
+    """
+    Container object for a bugzilla Group.
+
+    :arg bugzilla: Bugzilla instance that this Group belongs to.
+    Rest of the params come straight from Group.get()
+    """
+    def __init__(self, bugzilla, **kwargs):
+        self.bugzilla = bugzilla
+        self.__groupid = kwargs.get('id')
+
+        self.name = kwargs.get('name')
+        self.description = kwargs.get('description', self.name)
+        self.is_active = kwargs.get('is_active', False)
+        self.icon_url = kwargs.get('icon_url', None)
+        self.is_active_bug_group = kwargs.get('is_active_bug_group', None)
+
+        self.membership = kwargs.get('membership', [])
+        self.__member_emails = set()
+        self._refresh_member_emails_list()
+
+    ########################
+    # Read-only attributes #
+    ########################
+
+    # We make these properties so that the user cannot set them.  They are
+    # unaffected by the update() method so it would be misleading to let them
+    # be changed.
+    @property
+    def groupid(self):
+        return self.__groupid
+
+    @property
+    def member_emails(self):
+        return sorted(self.__member_emails)
+
+    def _refresh_member_emails_list(self):
+        """
+        Refresh the list of emails of the members of the group.
+        """
+        if self.membership:
+            for m in self.membership:
+                if "email" in m:
+                    self.__member_emails.add(m["email"])
+
+    def refresh(self, membership=False):
+        """
+        Update Group object with latest info from bugzilla
+        """
+        newgroup = self.bugzilla.getgroup(
+            self.name, membership=membership)
+        self.__dict__.update(newgroup.__dict__)
+        self._refresh_member_emails_list()
+
+    def members(self):
+        """
+        Retrieve the members of this Group from bugzilla
+        """
+        if not self.membership:
+            self.refresh(membership=True)
+        return self.membership
